#########################################################################
# mmucl_gui.tcl - graphical interfaces to core mmucl procedures
#
# Copyright (C) 1997-1998 Mark Patton
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

namespace eval mmucl {
    namespace export gui_init user_entry option_menu toolbar
    
    variable Mmucl
    array set Mmucl {
	cfg,echo        1
	cfg,echo_color  yellow
	cfg,keep_line   1
	cfg,hist_min    3
	cfg,toolbar     1

	hist_loc        1
    }

    trace variable Mmucl(cfg,toolbar) w mmucl::config_toolbar
    
    set Mmucl(lib_dir) $env(MMUCL_DIR)
}

# mmucl::gui_init --
#
#   Load in images, read in option databases

proc mmucl::gui_init {} {
    variable Mmucl
    global env tcl_platform

    lappend Mmucl(save_cmds) key
    
    # Load icons
    foreach image [glob [file join $Mmucl(lib_dir) images *.gif]] {
	image create photo [file tail $image] -file $image
    }
    
    # Specify an icon if the toplevel is minimized (unix only).
    if {$tcl_platform(platform) == "unix"} {
	toplevel .icon
	label .icon.l -image mmucl.gif -bd 0
	pack .icon.l
	wm iconwindow . .icon
    }
    
    # Load gui configuration.

    set conf [file join $Mmucl(lib_dir) Tk.conf]
    if {[catch {option readfile $conf} error]} {
	puts stderr "$conf: $error"
    }

    set conf [file join $Mmucl(rc_dir) Tk.conf]
    if {[file exists $conf]} {
	if {[catch {option readfile $conf} error]} {
	    puts stderr "$conf: $error"
	}
    }

    return
}

# mmucl::user_entry --
#
#   Set up an entry widget to parse input to mmucl.

proc mmucl::user_entry {w} {
    entry $w
    
    bind $w <Key-Return> {mmucl::meta_parse %W}
    bind $w <Key-Up> {mmucl::hist_scroll_up %W}
    bind $w <Key-Down> {mmucl::hist_scroll_down %W}
    
    # Toggle a debug command to eval script in the master interp
    bind $w <Control-Shift-Escape> {
	if {[string length [interp alias mmucl debug]]} {
	    interp alias mmucl debug {}
	} else {
	    interp alias mmucl debug {} uplevel #0
	}
    }
    
    # MCkey binds scripts to mmucl
    bindtags $w [list $w Entry mmucl]
    
    return $w 
}

# mmucl::toolbar --
#
#   Make a toolbar.

proc mmucl::toolbar {w} {
    frame $w
    
    button $w.con -command mmucl::tk_connect -image connect.gif 
    balloon_add $w.con "Connect to mud"
    button $w.load -image load.gif \
	-command {tk_openFile mmucl::load  -title {Load a File}}
    balloon_add $w.load "Load a file"
    button $w.send -image send.gif \
	-command {tk_openFile mmucl::MCtextin -title {Send a file to the mud}}
    balloon_add $w.send "Send a file"
    button $w.save -image save.gif \
	-command {tk_saveFile mmucl::MCsave -title {Save Current State}}
    balloon_add $w.save "Save state"
    button $w.char -image char.gif -command {mmucl::tk_edit char}
    balloon_add $w.char "Edit characters"
    button $w.alias -image alias.gif -command {mmucl::tk_edit alias}
    balloon_add $w.alias "Edit aliases"
    button $w.action -image action.gif -command {mmucl::tk_edit action}
    balloon_add $w.action "Edit actions"
    button $w.sub -image sub.gif -command {mmucl::tk_edit sub}
    balloon_add $w.sub "Edit subs"
    button $w.key -image bind.gif -command {mmucl::tk_edit key}
    balloon_add $w.key "Edit keys"

    pack $w.con $w.load $w.send $w.save $w.char $w.alias $w.action $w.sub \
	$w.key -side left 

    return $w
}

proc mmucl::load {file} {
    if {[catch {mmucl eval [list source $file]} error ] == 1} {
	report -error "$file: $error"
    }

    return
}

# mmucl::optionMenu --
#
#   Make a menu of mmucl options and return it.

proc mmucl::option_menu {w} {
    set var mmucl::Mmucl(cfg

    menu $w
    $w add checkbutton -label Reconnect -variable $var,reconnect)
    $w add checkbutton -label "Keep line" -variable $var,keep_line)
    $w add checkbutton -label Actions -variable $var,actions)
    $w add checkbutton -label Subs -variable $var,subs)
    $w add checkbutton -label "Local echo" -variable $var,echo)
    $w add cascade -label "Echo color" -menu $w.colors
    
    menu $w.colors
    foreach color {red green yellow blue magenta cyan grey} {
	$w.colors add radiobutton -label $color -value $color \
	    -variable $var,echo_color)
    }

    return $w
}

# mmucl::meta_parse --
#
#   A layer on top of parse that adds echoing, echo coloring, optionally
#   records input in history. Should be bound to an entry widget.

proc mmucl::meta_parse {w} {
    variable Mmucl

    set input [$w get]

    if {$Mmucl(cfg,echo)} {
	MCecho [MCcolor $input $Mmucl(cfg,echo_color)]
    }

    MCparse $input

    if {[string length $input] >= $Mmucl(cfg,hist_min)} {
	history add $input
	set Mmucl(hist_loc) 1
    }

    if {$Mmucl(cfg,keep_line)} {
	$w selection range 0 end
    } else {
	$w delete 0 end
    }

    return
}

# mmucl::hist_scroll_up --
#
#

proc mmucl::hist_scroll_up {w} {
    variable Mmucl

    if {$Mmucl(hist_loc) == 1} {
        set Mmucl(curline) [$w get]
    }

    if {[expr {1 - [history keep]}] < $Mmucl(hist_loc)} {
	$w delete 0 end
        incr Mmucl(hist_loc) -1
	$w insert end [history event $Mmucl(hist_loc)]
    }

    return
}

# mmucl::hist_scroll_down --
#
#

proc mmucl::hist_scroll_down {w} {
    variable Mmucl

    if {$Mmucl(hist_loc) == 1} {
	return
    }

    $w delete 0 end

    if {[incr Mmucl(hist_loc)] == 1} {
        set line $Mmucl(curline)
    } else {
        set line [history event $Mmucl(hist_loc)]
    }

    $w insert end $line

    return
}

# mmucl::tk_connect --
#
#   Dialog box to connect by selecting a defined character
#   or typing the location in.

proc mmucl::tk_connect {} {
    variable Mmucl
    
    array set chars $Mmucl(chars)

    if {[winfo exists .con]} {
	raise .con
	return
    }
    
    toplevel .con
    wm title .con "Connect"

    frame .con.f -relief groove -bd 2
    grid [label .con.f.hostl -text Host:]
    grid [entry .con.f.host] -row 0 -column 1
    grid [label .con.f.portl -text Port:] 
    grid [entry .con.f.port] -row 1 -column 1
    
    .con.f.host insert end $Mmucl(host)
    .con.f.port insert end $Mmucl(port)
    set Mmucl(tmpchar) ""

    # if characters defined, make a character selection area
    if {[array size chars]} {
	grid [label .con.f.lchar -text Char:]

	set names [lsort [array names chars]]
        set m [eval tk_optionMenu .con.f.char mmucl::Mmucl(tmpchar) $names]
	$m add separator
	$m add command -label None -command {set mmucl::Mmucl(tmpchar) ""}
	set Mmucl(tmpchar) ""

	grid .con.f.char -row 2 -column 1
	
	# Hack to update host and port on char selection.
	bind $m <ButtonRelease> {
	    .con.f.host delete 0 end
	    .con.f.port delete 0 end
	    
	    after 10 {
		if {$mmucl::Mmucl(tmpchar) != ""} {
		    .con.f.host insert 0 \
			[lindex [mmucl::MCchar set $mmucl::Mmucl(tmpchar)] 1]
		    .con.f.port insert 0 \
			[lindex [mmucl::MCchar set $mmucl::Mmucl(tmpchar)] 2]
		}
	    }
	}
    }
    
    frame .con.b
    pack [button .con.b.ok -text Connect \
	      -command {
		  if {![string length $mmucl::Mmucl(tmpchar)]} {
		      mmucl::MCconnect [.con.f.host get] [.con.f.port get]
		  } else {
		      mmucl::MCchar load $mmucl::Mmucl(tmpchar)
		  }
		  destroy .con}
	 ] -side left
    
    pack [button .con.b.quit -text Cancel -command {destroy .con}] -side left
  
    pack .con.f .con.b
    wm geometry .con +[winfo pointerx .]+[winfo pointery .]
    wm resizable .con 0 0

    return
}

# mmucl::tk_edit --
#
#   Pops up a dialog box to let the user edit the given array.  

proc mmucl::tk_edit {proc} {
    set list 0
    switch $proc {
	action {
	    set labels {Pattern: Script:}
	    set title "Edit Actions"
	} alias {
	    set labels {Name: Script:}
	    set title "Edit Aliases"
	} sub {
	    set labels {Pattern: Subspec:}
	    set title "Edit Subs"
	} key {
	    set labels {Event: Script:}
	    set title "Edit Keys"
	} char {
	    set labels {Name: Mud: Host: Port: Login:}
	    set title "Edit Characters"
	    set list 1
	}
    }

    set w .$proc

    if {[winfo exists $w]} {
	raise $w
	return
    }
  
    toplevel $w
    wm title $w $title

    form $w.form $labels mmucl::MC$proc $list
    pack $w.form -side top -pady 3
 
    frame $w.b
    pack [button $w.b.help -text Help -command [list mmucl::MChelp]] \
  	[button $w.b.exit -text Exit -command [list destroy $w]] \
	-side left
    pack $w.b -side bottom
    
    wm geometry $w +[winfo pointerx .]+[winfo pointery .]
    wm resizable $w 0 0

    return
}

# mmucl::MCkey --
#
#   Let the user bind commands to an entry. The command are executed in
#   the mmucl interp.

proc mmucl::MCkey {args} {
    variable Mmucl

     set syntax {
	 set    {{2 3} {}  "key ?script?"}
	 names  {{2}   {*} "?pattern?"}
	 print  {{2}   {*} "?pattern?"}
	 delete {{2}   {}  "pattern"}
     }
    
    set option [MCgetopt action $syntax $args]
    
    foreach name [bind mmucl] {
	set script [bind mmucl $name]
	set script [string range $script 25 \
			[expr {[string length $script] - 104}]]

	set keys($name) $script
    }

    if {$option == "set"} {
	set key [lindex $args 1]

	if {$n == 3} {
	    if {[catch {
	    bind mmucl $key "
if {\[catch {mmucl eval {[lindex $args 2]}} mmucl::Mmucl(key_error)\] == 1} {
    mmucl::report -error \"key $key: \$mmucl::Mmucl(key_error)\"
}"} error]} {
    report -error "key: $error"
}

	} elseif {![info exist keys($key)]} {
	    report_error "no such key"
	} else {
	    return $keys($key)
	}
    } elseif {$option == "names"} {
        return [array names keys [lindex $args 1]]
    } elseif {$option == "delete"} {
	foreach key [array names keys [lindex $args 1]] {
	    bind mmucl $key  {}
	}
    } else {
	foreach key [array names keys [lindex $args 1]] {
	    MCecho "\"$key\" triggers {$keys($key)}"
	}
    }

    return
}

proc mmucl::MChelp {{subject ""}} {
    variable Mmucl
    global tki tkiEmbed env tcl_platform

    if {![info exists tki]} {
	set tkiEmbed 1

	# Find tkinfo somewhere in the users path
	if {$tcl_platform(platform) == "unix"} {
	    set path [split $env(PATH) :]
	} elseif {$tcl_platform(platform) == "windows"} {
	    set path [split $env(PATH) \;]
	} else {
	    set path [pwd]
	}

	foreach dir $path {
	    if {[file exists [file join $dir tkinfo]]} {
		uplevel #0 [list source [file join $dir tkinfo]]
		break
	    }
	}
	
	if {![info exists tki]} {
	    report_error "tkinfo not in path"
	    return
	}
    }
    
    tkiWinShow (mmucl)$subject

    return
}

# Modify the command line . Widget
# name shouldn't be hardcoded...

proc mmucl::MCcline {args} {
    set syntax {
	delete {{2 3} {} "first ?last?"}
	get {{1} {} {}} 
	insert {{3} {} "index string"}
     }

    MCgetopt cline $syntax $args 
    eval .mmucl_input $args	
}

# Invoked by a trace on cfg,toolbar
# Again widget shouldn't be hardcoded.

proc mmucl::config_toolbar {array key op} {
    variable Mmucl

    if {$Mmucl(cfg,toolbar)} {
	grid .toolbar
    } else {
	grid remove .toolbar
    }
}

bind Entry <Control-u> {
    %W delete 0 insert
}

# Misc procedures.

# Dialog for giving an existing file as an arg to a command

proc tk_openFile {proc args} {
    set filename  [eval tk_getOpenFile $args]
    
    if {[string length $filename]} {
	 $proc $filename
     }

    return
}

# Dialog for giving a file to a command as an arg

proc tk_saveFile {proc args} {
    set filename [eval tk_getSaveFile $args]
   
    if {[string length $filename]} {
	$proc $filename
    }

    return
}

# Execute a proc when given some result from tk_messageBox

proc tk_dialogBox {proc on_result args} {
    set result [eval tk_messageBox $args]

    if {$result == $on_result} {
	$proc
    }

    return
}
